from django.db import models
from django.utils.translation import gettext_lazy as _
from django.contrib.auth import get_user_model
# Create your models here.
from tinymce.fields import TinyMCEField
from baykeshop.common.validators import validate_phone
from baykeshop.common.models import BaseModelMixin
# from baykeshop.apps.stats.models import BaykeDataStats


class BaykeShopCategory(BaseModelMixin):
    """Model definition for BaykeShopCategory."""
    name = models.CharField(_("名称"), max_length=50)
    parent = models.ForeignKey(
        "self",
        on_delete=models.CASCADE,
        blank=True,
        null=True,
        verbose_name=_("父类")
    )
    icon = models.ImageField(
        _("图标"),
        upload_to="shop/category",
        max_length=200,
        blank=True,
        null=True
    )
    figure = models.ImageField(
        _("形象图"),
        upload_to="shop/category",
        max_length=200,
        blank=True,
        null=True
    )
    sort = models.PositiveSmallIntegerField(default=1, verbose_name=_("排序"))
    is_nav = models.BooleanField(default=True, verbose_name=_("推荐"))
    status = models.BooleanField(default=True, verbose_name=_("状态"))
    # TODO: Define fields here

    class Meta:
        """Meta definition for BaykeShopCategory."""
        ordering = ['sort']
        verbose_name = _('分类')
        verbose_name_plural = verbose_name

    def __str__(self):
        """Unicode representation of BaykeShopCategory."""
        return self.name


class BaykeShopBrand(BaseModelMixin):
    """Model definition for BaykeShopBrand."""
    name = models.CharField(max_length=100, verbose_name=_("品牌名称"))  # 品牌名称
    desc = models.CharField(blank=True, default="",
                            max_length=150, verbose_name=_("品牌描述"))  # 品牌描述
    logo = models.ImageField(
        upload_to='brand_logos/', null=True, blank=True, verbose_name=_("品牌标志"))  # 品牌标志
    # TODO: Define fields here

    class Meta:
        """Meta definition for BaykeShopBrand."""
        ordering = ['-add_date']
        verbose_name = _('品牌')
        verbose_name_plural = verbose_name

    def __str__(self):
        """Unicode representation of BaykeShopBrand."""
        return self.name


class BaykeShopSPU(BaseModelMixin):
    """Model definition for BaykeShopSPU."""
    title = models.CharField(_("标题"), max_length=100)
    subtitle = models.CharField(
        _("副标题"), max_length=150, blank=True, default="")
    category = models.ManyToManyField(
        BaykeShopCategory, blank=True, verbose_name=_("商品分类"))
    brand = models.ForeignKey(
        BaykeShopBrand, on_delete=models.DO_NOTHING, verbose_name=_("商品品牌"))
    content = TinyMCEField(_("详情"))
    unit = models.CharField(_("单位"), max_length=50, blank=True, default="")
    shipping_price = models.DecimalField(
        max_digits=10, decimal_places=2, blank=True, default=0, verbose_name=_("运费"))  # 运费
    status = models.BooleanField(_("状态"), default=True)

    # TODO: Define fields here

    class Meta:
        """Meta definition for BaykeShopSPU."""
        ordering = ['-add_date']
        verbose_name = _('商品')
        verbose_name_plural = verbose_name

    def __str__(self):
        """Unicode representation of BaykeShopSPU."""
        return self.title

    @classmethod
    def get_hots(cls):
        # 热销商品，按销量统计
        return cls.objects.filter(status=True).alias(
            sales=models.Sum('baykeshopsku__sales')
        ).annotate(sales=models.F('sales')).order_by('-sales')


class BaykeShopSKU(BaseModelMixin):
    """Model definition for BaykeShopSKU."""
    spu = models.ForeignKey(
        BaykeShopSPU, on_delete=models.CASCADE, verbose_name=_("SPU"))
    img = models.ImageField(_("主图"), upload_to="shop/sku", max_length=200)
    price = models.DecimalField(_("售价"), max_digits=10, decimal_places=2)  # 售价
    cost_price = models.DecimalField(
        _("成本价"), max_digits=10, decimal_places=2)  # 成本价
    discount_price = models.DecimalField(
        _("划线价"), max_digits=10, decimal_places=2)  # 划线价
    stock = models.PositiveSmallIntegerField(
        default=0, verbose_name=_("库存"))  # 库存
    sales = models.PositiveSmallIntegerField(
        default=0, verbose_name=_("销量"))  # 销量
    code = models.CharField(max_length=20, blank=True,
                            default="", verbose_name=_("商品编码"))  # 编码
    volume = models.DecimalField(
        max_digits=10, decimal_places=2, blank=True, default=0, verbose_name=_("体积"))  # 体积
    weight = models.DecimalField(
        max_digits=10, decimal_places=2, blank=True, default=0, verbose_name=_("重量"))  # 重量
    specs = models.JSONField(default=dict, blank=True, verbose_name=_("规格数据"))

    # TODO: Define fields here

    class Meta:
        """Meta definition for BaykeShopSKU."""
        ordering = ['price']
        verbose_name = _('商品规格')
        verbose_name_plural = verbose_name

    def __str__(self):
        """Unicode representation of BaykeShopSKU."""
        return self.spu.title


class BaykeShopSpec(BaseModelMixin):
    """Model definition for BaykeShopSpec."""
    name = models.CharField(_("名称"), max_length=50)

    # TODO: Define fields here

    class Meta:
        """Meta definition for BaykeShopSpec."""
        ordering = ['-add_date']
        verbose_name = _('规格')
        verbose_name_plural = verbose_name

    def __str__(self):
        """Unicode representation of BaykeShopSpec."""
        return self.name


class BaykeShopSpecValue(BaseModelMixin):
    """Model definition for BaykeShopSpecValue."""
    spec = models.ForeignKey(
        BaykeShopSpec, on_delete=models.CASCADE, verbose_name=_("规格"))
    value = models.CharField(_("规格值"), max_length=50)

    # TODO: Define fields here

    class Meta:
        """Meta definition for BaykeShopSpecValue."""
        ordering = ['add_date']
        verbose_name = _('规格值')
        verbose_name_plural = verbose_name

    def __str__(self):
        """Unicode representation of BaykeShopSpecValue."""
        return f"{self.spec.name}:{self.value}"


class BaykeShopSPUAtlas(BaseModelMixin):
    """Model definition for BaykeShopSPUAtlas."""
    spu = models.ForeignKey(
        BaykeShopSPU, on_delete=models.CASCADE, verbose_name=_("商品"))
    img = models.ImageField(
        _("商品轮播图"),
        upload_to="spu/%Y/%m",
        height_field="width",
        width_field="height",
        max_length=200
    )
    width = models.PositiveSmallIntegerField(
        blank=True, default=500, verbose_name=_("图片宽度"))
    height = models.PositiveSmallIntegerField(
        blank=True, default=500, verbose_name=_("图片高度"))
    sort = models.PositiveSmallIntegerField(default=1, verbose_name=_("排序"))
    status = models.BooleanField(default=True, verbose_name=_("状态"))

    # TODO: Define fields here

    class Meta:
        """Meta definition for BaykeShopSPUAtlas."""
        ordering = ['sort']
        verbose_name = _('商品轮播图')
        verbose_name_plural = verbose_name

    def __str__(self):
        """Unicode representation of BaykeShopSPUAtlas."""
        return self.img.name


class BaykeShopCart(BaseModelMixin):
    """Model definition for BaykeShopCart."""
    owner = models.ForeignKey(
        get_user_model(), on_delete=models.CASCADE, verbose_name=_("用户"))
    sku = models.ForeignKey(
        BaykeShopSKU, on_delete=models.CASCADE, verbose_name=_("规格"))
    num = models.PositiveSmallIntegerField(default=1, verbose_name=_("数量"))

    # TODO: Define fields here

    class Meta:
        """Meta definition for BaykeShopCart."""
        ordering = ['-add_date']
        verbose_name = _('购物车')
        verbose_name_plural = verbose_name
        constraints = [
            models.UniqueConstraint('owner', 'sku', name='unique_owner_sku'),
        ]

    def __str__(self):
        """Unicode representation of BaykeShopCart."""
        return self.sku.spu.title

    @classmethod
    def get_cart_count(cls, user) -> dict:
        # 当前用户的购物车商品数量
        from django.db.models import Sum
        counts = cls.objects.filter(owner=user).aggregate(Sum('num'))
        return counts.get('num__sum') if counts.get('num__sum') else 0


class BaykeShopOrder(BaseModelMixin):
    """Model definition for BaykeShopOrder."""
    class PayMethod(models.IntegerChoices):
        ALIPAY = 1, _("支付宝支付")
        WXPAY = 2, _("微信支付")
        BALANCE = 3, _("余额支付")

        __empty__ = _("(Unknown)")

    class OrderStatus(models.IntegerChoices):
        UNPAID = 1, _("待付款")
        UNSHIP = 2, _("待发货")
        UNGOODS = 3, _("待收货")
        UNCOMMENT = 4, _("待评价")
        DONE = 5, _("已完成")
        CLOSED = 6, _("已关闭")
        REFUND = 7, _("退款中")

    owner = models.ForeignKey(
        get_user_model(), on_delete=models.CASCADE, verbose_name=_("用户"))
    status = models.PositiveSmallIntegerField(
        choices=OrderStatus.choices, default=OrderStatus.UNPAID, verbose_name=_("订单状态"))
    paymethod = models.PositiveSmallIntegerField(
        choices=PayMethod.choices, blank=True, verbose_name=_("支付方式"), null=True)
    order_sn = models.CharField(_("订单号"), max_length=100, blank=True)
    total_price = models.DecimalField(
        _("总价"), max_digits=10, decimal_places=2, blank=True, default=0)
    mark = models.CharField(_("订单备注"), max_length=150, blank=True, default="")
    name = models.CharField("签收人", max_length=50, blank=True, default="")
    phone = models.CharField("手机号", max_length=11, blank=True, default="", validators=[validate_phone])
    email = models.EmailField("邮箱", blank=True, default="", max_length=50)
    address = models.CharField("收货地址", max_length=200, blank=True, default="")
    pay_time = models.DateTimeField(
        null=True, blank=True, verbose_name="支付时间", help_text="支付时间", editable=False)
    # TODO: Define fields here

    class Meta:
        """Meta definition for BaykeShopOrder."""
        ordering = ['-add_date']
        verbose_name = _('订单')
        verbose_name_plural = verbose_name
        constraints = [
            models.UniqueConstraint(
                'owner', 'order_sn', name='unique_owner_order'),
        ]
        permissions = [
            ("send_out_goods", "send out goods"),
        ]

    def __str__(self):
        """Unicode representation of BaykeShopOrder."""
        return self.order_sn

    def save(self, *args, **kwargs):
        if not self.order_sn:
            self.order_sn = self.generate_order_sn()
        super().save(*args, **kwargs)

    def generate_order_sn(self):
        # 当前时间 + userid + 随机数
        from random import Random
        from django.utils import timezone
        random_ins = Random()
        order_sn = "{time_str}{user_id}{ranstr}".format(
            time_str=timezone.now().strftime("%Y%m%d%H%M%S"),
            user_id=self.owner.id,
            ranstr=random_ins.randint(10, 99))
        return order_sn


class BaykeShopOrderSKU(BaseModelMixin):
    """Model definition for BaykeShopOrderSKU."""
    order = models.ForeignKey(
        BaykeShopOrder, on_delete=models.CASCADE, verbose_name=_("订单"))
    sku = models.ForeignKey(
        BaykeShopSKU, on_delete=models.PROTECT, verbose_name=_("商品规格"))
    count = models.PositiveSmallIntegerField(default=1, verbose_name=_("数量"))
    sku_json = models.JSONField(
        verbose_name=_("商品快照"), blank=True, default=dict)
    is_commented = models.BooleanField(default=False, verbose_name="是否已评价")

    # TODO: Define fields here

    class Meta:
        """Meta definition for BaykeShopOrderSKU."""
        ordering = ['-add_date']
        verbose_name = _('订单商品')
        verbose_name_plural = verbose_name

    def __str__(self):
        """Unicode representation of BaykeShopOrderSKU."""
        return self.sku.spu.title

    def save(self, *args, **kwargs):
        self.sku_json = {
            "title": self.sku.spu.title,
            "subtitle": self.sku.spu.subtitle,
            "content": self.sku.spu.content,
            "img": self.sku.img.url,
            "code": self.sku.code,
            "price": self.sku.price.to_eng_string(),
            "cost_price": self.sku.cost_price.to_eng_string(),
            "discount_price": self.sku.discount_price.to_eng_string(),
            "unit": self.sku.spu.unit,
            "shipping_price": self.sku.spu.shipping_price.to_eng_string(),
            "specs": {key: value for key, value in self.sku.specs.items() if not key.isdigit()}
        }
        super().save(*args, **kwargs)


class BaykeAddress(BaseModelMixin):
    """ 收货地址 """
    owner = models.ForeignKey(get_user_model(), on_delete=models.CASCADE, verbose_name="用户")
    name = models.CharField("签收人", max_length=50)
    phone = models.CharField("手机号", max_length=11)
    email = models.EmailField("邮箱", blank=True, default="", max_length=50)
    province = models.CharField(max_length=150, verbose_name="省")
    city = models.CharField(max_length=150, verbose_name="市")
    county = models.CharField(max_length=150, verbose_name="区/县")
    address = models.CharField(max_length=150, verbose_name="详细地址")
    is_default = models.BooleanField(default=False, verbose_name="设为默认")
    
    class Meta:
        ordering = ['-add_date']
        verbose_name = "收货地址"
        verbose_name_plural = verbose_name
       
    def __str__(self):
        return f'{self.name} {self.address}'



from django.db import models
from django.utils.translation import gettext_lazy as _
from django.contrib.auth import get_user_model
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType


from baykeshop.common.queryset import QuerySet


User = get_user_model()


class Manger(models.Manager):
    """ 管理器 """
    
    def get_queryset(self) -> QuerySet:
        return QuerySet(model=self.model, using=self._db, hints=self._hints)
    
    def body(self):
        return self.get_queryset().body()
    
    def nobody(self):
        return self.get_queryset().nobody()
    
    def fakedelete(self):
        return self.get_queryset().fakedelete()
    
    def regain(self):
        return self.get_queryset().regain()


class BaseModelMixin(models.Model):
    # TODO
    
    add_date = models.DateTimeField(auto_now_add=True, verbose_name="添加时间")
    pub_date = models.DateTimeField(auto_now=True, verbose_name="修改时间")
    is_delete = models.BooleanField(default=False, editable=False, verbose_name="删除标记")
    
    objects = Manger()

    class Meta:
        abstract = True
        
        
class ContentTypeAbstract(BaseModelMixin):
    """ 模型的通用关系 """
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    object_id = models.PositiveIntegerField(blank=True, null=True)
    tag = models.CharField(_('标记'), max_length=200, blank=True, default="")
    content_object = GenericForeignKey('content_type', 'object_id')

    class Meta:
        abstract = True
        indexes = [
            models.Index(fields=["content_type", "object_id"]),
        ]




'''
@file            :validators.py
@Description     :验证器
@Date            :2023/07/17 17:11:50
@Author          :幸福关中 && 轻编程
@version         :v1.0
@EMAIL           :1158920674@qq.com
@WX              :baywanyun
'''


import re

from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _
from baykeshop.conf import bayke_settings


def validate_phone(value):
    # 中国区手机号验证
    reg = re.compile(bayke_settings.REGEX_PHONE)
    if not reg.search(value):
        raise ValidationError(
            _("%(value)s 格式有误"),
            params={"value": value},
        )
 

def validate_count(value):
    # 验证列表的长度不超过10个
    if isinstance(value, list):
        if len(value) > 10:
            raise ValidationError(
                _("%(value)s is max length 10"),
                params={"value": value},
            )


from typing import Any
from django.contrib import admin
from django.contrib import messages
from django.http.request import HttpRequest
from django.utils.translation import ngettext
# Register your models here.
from baykeshop.common.options import ModelAdmin, TabularInline
from .models import (
    BaykeArticleCategory, BaykeArticleContent, BaykeArticleTags
)

class BaykeArticleCategoryInline(TabularInline):
    '''Stacked Inline View for BaykeArticleCategory'''

    model = BaykeArticleCategory
    min_num = 1
    max_num = 100
    extra = 1
    exclude = ('desc', 'keywords')


@admin.register(BaykeArticleCategory)
class BaykeArticleCategoryAdmin(ModelAdmin):
    '''Admin View for BaykeArticleCategory'''

    list_display = ('id', 'name', 'parent', 'sort', 'status', 'add_date')
    list_display_links = ('id', 'name')
    list_editable = ('sort', )
    search_fields = ('name',)
    exclude = ('parent',)
    inlines = [BaykeArticleCategoryInline]

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
       if db_field.name == 'parent':
           kwargs['queryset'] = BaykeArticleCategory.objects.filter(parent__isnull=True)
       return super().formfield_for_foreignkey(db_field, request, **kwargs)
    
    def has_change_permission(self, request, obj=None):
        if obj and obj.parent:
            return False
        return super().has_change_permission(request, obj)
    
    def get_exclude(self, request, obj=None):
        if obj and obj.parent:
            self.exclude = None
        return super().get_exclude(request, obj)


@admin.register(BaykeArticleContent)
class BaykeArticleContentAdmin(ModelAdmin):
    '''Admin View for BaykeArticleContent'''

    list_display = ('id', 'title', 'category', 'status', 'add_date', 'pub_date')
    list_display_links = ('id', 'title')
    list_filter = ('category',)
    search_fields = ('title',)
    filter_horizontal = ('tags',)
    date_hierarchy = 'add_date'
    actions = ['make_draft']
    
    def formfield_for_foreignkey(self, db_field, request, **kwargs):
       if db_field.name == 'category':
           kwargs['queryset'] = BaykeArticleCategory.objects.exclude(parent__isnull=True)
       return super().formfield_for_foreignkey(db_field, request, **kwargs)
    
    @admin.action(description="所选内容批量设为草稿", permissions=["change"])
    def make_draft(self, request, queryset):
        updated = queryset.update(status=0)
        self.message_user(
            request,
            ngettext(
                "%d story was successfully marked as draft.",
                "%d stories were successfully marked as draft.",
                updated,
            )
            % updated,
            messages.SUCCESS,
        )

    
@admin.register(BaykeArticleTags)
class BaykeArticleTagsAdmin(ModelAdmin):
    '''Admin View for BaykeArticleTags'''

    list_display = ('id', 'name', 'add_date')
    list_display_links = ('id', 'name')


from typing import Any, Dict
from django.views.generic import ListView, DetailView
from django.views.generic.detail import SingleObjectMixin
from django.views.generic.dates import MonthArchiveView
# Create your views here.
from .models import BaykeArticleContent, BaykeArticleCategory, BaykeArticleTags


class BaykeArticleContentListView(ListView):
    """列表页
    """
    queryset = BaykeArticleContent.objects.exclude(status=0)
    template_name = "article/list.html"
    paginate_by = 20
    paginate_orphans = 2
    extra_context = {"title": "全部文章"}


class BaykeArticleCategoryDetailView(SingleObjectMixin, BaykeArticleContentListView):
    """分类列表页
    """
    
    def get(self, request, *args, **kwargs):
        self.object = self.get_object(queryset=BaykeArticleCategory.objects.all())
        return super().get(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['title'] = self.object.name
        return context

    def get_queryset(self):
        return self.object.baykearticlecontent_set.filter(status=1)
    

class BaykeArticleContentDetailView(DetailView):
    """文章详情页
    """
    model = BaykeArticleContent
    template_name = "article/detail.html"
    context_object_name = "article"

    def get_context_data(self, **kwargs: Any) -> Dict[str, Any]:
        context = super().get_context_data(**kwargs)
        context['article_next'] = self.get_object().next_article
        context['article_previous'] = self.get_object().previous_article
        context['title'] = self.get_object().title
        return context


class BaykeArticleContentMonthArchiveView(MonthArchiveView, BaykeArticleContentListView):
    """ 文章归档 """
    date_field = "add_date"
    allow_future = True
    month_format = "%m"
    
    def get_context_data(self, **kwargs: Any) -> dict[str, Any]:
        context = super().get_context_data(**kwargs)
        context['title'] = context.get('month')
        return context
    

class BaykeArticleTagsToArticleListView(SingleObjectMixin, BaykeArticleContentListView):
    # 文章标签列表页
    
    def get(self, request, *args, **kwargs):
        self.object = self.get_object(BaykeArticleTags.objects.all())
        return super().get(request, *args, **kwargs)
    
    def get_queryset(self):
        return self.object.baykearticlecontent_set.all()
    
    def get_context_data(self, **kwargs: Any) -> dict[str, Any]:
        context = super().get_context_data(**kwargs)
        context['title'] = self.object.name
        return context




#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@文件    :queryset.py
@说明    :自定义管理器
@时间    :2023/06/30 14:13:20
@作者    :幸福关中&轻编程
@版本    :1.0
@微信    :baywanyun
'''

from django.db.models.query import QuerySet as BaseQueryset


class QuerySet(BaseQueryset):
    
    def nobody(self):
        # 查询已被伪删除的数据
        return self.filter(is_delete=True)
    
    def body(self):
        # 查询未被伪删除的数据
        return self.filter(is_delete=False)
    
    def fakedelete(self):
        # 伪删除
        return self.body().update(is_delete=True)
    
    def regain(self):
        # 恢复伪删除数据
        return self.nobody().update(is_delete=False)